function [vBeta, vErr_at_peaks, hWaitbar] = find_modes(oGd, pos, k0, min_beta, max_beta,...
    tol, bPlot, nSampMono, absMinErr, complex, maxNeffI, sMethod, varargin)
% [Neff, error] = FIND_MODES(oGd, positions, k0, minNeff, maxNeff, tol, bShow, maxN, maxErr, complex, minNeffI, method)
%   Finds modes for the geometry in oGd.
%   Sources and testing points are distributed according to positions.
%   Free-space wave number: k0.
%   Range of effective index: (minNeff, maxNeff)
%   Tolerance for effective index: tol
%   Show search progress: bShow.
%   Max. Number of points in a monotonic interval (Nmax): maxN
%   Max. acceptable error: maxErr
%   Searches for complex effective indices if complex ~= 0.
%   For proper behavior at infinity, complex = 1. For improper (i.e. leaky
%     modes), complex = 2.
%   Min. imaginary part: minNeffI. If minNeffI = 0, the algorithm determines
%     the search region from the shape of the error on the real line.
%   Eigenvalue determination method: method, can be either 'direct', or
%     'indirect'. Indirect is slower, but can avoid false minima which may
%     occur with the direct method (false minima may be caused by poor source
%     location)
global bSkip bCancel
bSkip  = 0;
bCancel = 0;
if length(varargin) == 2
    bFromGui = 1;
    hWaitbar = varargin{2};
    hButton = findobj(get(hWaitbar, 'children'), 'tag', 'uiWBButton');
    bMakeNewWB = isempty(hWaitbar);
else
    bFromGui = 0;
    bMakeNewWB = 1;
    if bPlot
        hFig = gcf;
    end
end

if bMakeNewWB
    hWaitbar = waitbar(0,'searching for modes...','CreateCancelBtn', @cancel_callback, 'visible', 'off');
    hButton = uicontrol(hWaitbar, 'style','pushbutton');
    set(hButton, 'callback', @skip_callback)
    set(hButton, 'tag', 'uiWBButton')
end
if bFromGui
    hLog = findobj('tag', 'text', 'style', 'listbox');
    set(hLog, 'visible', 'off')
    hPanel = findobj('tag', 'uiPanel');
    set(hPanel, 'visible', 'on')
    hFig = findobj('tag', 'uiSmtgui');
    set(hFig, 'units', 'points')
    panel_pos = get_abs_pos(hFig, hPanel);
    set(hWaitbar, 'color', get(hFig, 'color'))
end
set(hWaitbar, 'units', 'points')
bar_pos = get(hWaitbar, 'position');
set(get(hWaitbar, 'children'), 'units', 'normalized')
if bFromGui
    bar_pos(3) = panel_pos(3)*0.7;
else
    bar_pos(3) = 379.5750;
end
if bFromGui
    pos_for_bar = center_pos(bar_pos, panel_pos);
    pos_for_bar(4) = panel_pos(4)-35;
    pos_for_bar(2) = panel_pos(2)+5;
else
    pos_for_bar = bar_pos;
end
set(hWaitbar, 'position', pos_for_bar);
set(hButton, 'string', 'Jump to Fine Search...')
set(hWaitbar, 'units', 'points')
set(hButton, 'units', 'pixels')
set(hButton,'position', [235 11 140 24])

if bMakeNewWB
    set(hWaitbar, 'visible', 'on')
end
if bPlot
    if ~isempty(varargin)
        hAx = varargin{1};
    else
        if ~bFromGui
            figure(hFig)
            clf
        end
        hAx = axes;
        cla(hAx);
    end
    set(gcf, 'DoubleBuffer','on')
else
    hAx = [];
end
fun = @(x) smt_solve(x, oGd, pos, k0, sMethod);
bFirstTime = 1;
[vMin, vMax, vNeff_full, vErr_full] = find_modes_rec(min_beta, max_beta, 0,...
    0, fun, nSampMono, bPlot, absMinErr, hAx, hWaitbar, bFirstTime);
if isnan(vMin)
    clean_up
    vBeta = -1;
    vErr_at_peaks = [];
    return
end
iMax = local_maxima(vErr_full);
vMax = vNeff_full(iMax);
iMin = local_maxima(1./vErr_full);
vMin = vNeff_full(iMin);
if isempty(vMin)
    vBeta = [];
    vErr_at_peaks = [];
    clean_up
    return
end
set(hButton, 'string', 'Finish now')
set(hButton, 'position', get(hButton, 'position').*[1.3 1 0.5 1])
bSkip = 0;
[vBeta, vErr_at_peaks] = find_degs(vMin, vMax, min_beta, max_beta, tol,...
    oGd, pos, k0, vNeff_full, vErr_full, absMinErr, complex, nSampMono, bPlot, sMethod, hAx, hWaitbar, maxNeffI);
clean_up

    function skip_callback(varargin)
        bSkip = 1;
    end

    function cancel_callback(varargin)
        bCancel = 1;
    end

    function clean_up
        if ~bFromGui
            delete(hWaitbar)
        end
    end
end
%#function smt_solve
