function curve =  primitive(sType, varargin)
% curve = PRIMITIVE(shape, ...) creates a curve object centered at the origin.
% shape is either 'circle', 'ellipse', 'cookie', or 'super_ellipse'.
% When shape is 'circle', the next argument is the radius
% When shape is 'ellipse', the arguments are the X-semiaxis and the ratio of
%   the Y-semiaxis to the X-semiaxis.
% When shape is 'cookie', the shape is defined in polar coordinates
%   by the formula: r(phi) = R*(1+m*cos(n*phi))
%   The order of arguments is R, m, and n.
% When shape is 'super_ellipse', the shape is a super ellipse (which
%    approximates a rectangle. The X dimension is the first argument, the
%    ratio of Y dimension to X dimension is the second argument, and the last
%    argument is n, which determines the sharpness of the corners.
% The field 'param' of the output curve holds the list of parameters used to
% define the curve.
switch lower(sType)
    case 'circle'
        nPoints = 50;
        R = varargin{1};
        oShape = simple_shape(sType, nPoints, R);
        x = oShape.x;
        y = oShape.y;
        x(end+1) = x(1);
        y(end+1) = y(1);
        tag = 'circle';
        curve.param = R;
    case 'ellipse'
        nPoints = 50;
        R = varargin{1};
        ell = varargin{2};
        oShape = simple_shape(sType, nPoints, [R, ell]);
        x = oShape.x;
        y = oShape.y;
        x(end+1) = x(1);
        y(end+1) = y(1);
        curve.param = [R, ell];
        tag = 'ellipse';
    case 'super_ellipse'
        r = varargin{1};
        ell = varargin{2};
        n = varargin{3};
        curve.param = [r, ell, n];
        nPoints = min(50*n, 200);
        phi = linspace(0, 2*pi - 2*pi/nPoints, nPoints);
        phi = phi+pi/nPoints;
        R = r*(abs(cos(phi)).^(2*n) + abs(sin(phi)).^(2*n)).^(-1/2/n);
        x = R.*cos(phi);
        y = ell*R.*sin(phi);
        x(end+1) = x(1);
        y(end+1) = y(1);
    case 'fused_coupler'
        delta = varargin{1};
        mu = varargin{2};
        b = varargin{3};
        curve.param = [delta, mu, b];
        nPoints = 200;
        phi = linspace(0, 2*pi - 2*pi/nPoints, nPoints);
        phi = phi+pi/nPoints;
        R = delta*(1+cos(2*phi))+1;
        x = mu*b*R.*cos(phi);
        y = b*R.*sin(phi);
        x(end+1) = x(1);
        y(end+1) = y(1);
        tag = 'fused_coupler';
    case 'cookie'
        nPoints = 50;
        phi = linspace(0, 2*pi - 2*pi/nPoints, nPoints);
        phi = phi+pi/nPoints;
        R = varargin{1};
        mod_amp = varargin{2};
        n = varargin{3};
        curve.param = [R, mod_amp, n];
        r = R*(1+mod_amp*cos(n*phi));
        x = r.*cos(phi);
        y = r.*sin(phi);
        x(end+1) = x(1);
        y(end+1) = y(1);
        tag = 'cookie';
    otherwise
        error('primitive: Unknown primitive type')
end
points = [x; y];
cs = cscvn(points);
N = 500;
vT = linspace(cs.breaks(1), cs.breaks(end), N);
mF = ppval(cs, vT);
xline = mF(1,:);
yline = mF(2,:);
dt = (vT(2)-vT(1));
dydt = diff(yline)/dt;
dxdt = diff(xline)/dt;
curve.length = sum(sqrt(dydt.^2 + dxdt.^2))*dt;
curve.iCurve = 1;
curve.cs = cs;
curve.max_x = max(xline);
curve.max_y = max(yline);
curve.min_x = min(xline);
curve.min_y = min(yline);
curve.xc = 0;
curve.yc = 0;
curve.tag = tag;
curve = orderfields(curve);