function [vErr, varargout] = smt_solve(Neff, oGd, pos, k0, sMethod, varargin)
% error = SMT_SOLVE(Neff, oGd, positions, k0, method)
% Solves the geometry defined in oGd.
% Sources and testing points are distributed according to positions.
% Free-space wave number: k0.
% Effective index: Neff, may be a vector
% Eigenvalue determination method: method, can be either 'direct', or
% 'indirect'. Indirect is slower, but can avoid false minima which may
% occur with the direct method (false minima may be caused by poor source
% location)
% Returns the error in continuity conditions.
% [error, solution] = SMT_SOLVE(Neff, oGd, pos, k0, method)
% Returns the vector of source amplitudes as well.
% If Neff is a vector, solution is a matrix. Each column of the matrix
% corresponds to one entry of the Neff vector.
% If Neff is complex, the function must be called with one more argument:
% [error, solution] = SMT_SOLVE(Neff, oGd, pos, k0, method, isImproper).
% isImproper determines whether the field obeys the radiation condition.
persistent guess
if length(varargin) == 2
    Neff = Neff(1) + j*Neff(2);
end
vBeta = Neff*k0;
vErr = zeros(1,length(vBeta));
mSol = zeros(2*sum([pos.sources.n]), length(vBeta));
if ~isreal(Neff)
    isImproper = varargin{1};
else
    isImproper = 0; % if Neff is real these values do not make any difference
end
for ind = 1:length(vBeta)
    beta = vBeta(ind);
    [mZ, mBNegate] = imp_matrix(oGd, pos, k0, beta, isImproper);
    if (length(guess) ~= size(mZ, 2))
        guess = [];
    end
    [err, guess] = non_trivial(mZ, mBNegate, sMethod, guess);
%    vErr(ind) = bc_error(mZ, mBNegate, guess);
    vErr(ind) = min(err, 0.4);
    if nargout == 2
        mSol(:,ind) = guess(:);
    end
end
if nargout == 2
    varargout{1} = mSol;
end
